/* ============================================================================================================================================= /
 * LIBRARY:         Gestione GLCD
 * FILE:            GLCD_KS0108B.h
 * LAST REV:        1.3
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         22/12/2014
 * UPDATED:
 *
 * DEPENDENCE:      Richiede libreria Delay.c
 *                  definire la frequenza del clock utilizzato #define XTAL_FREQ
 * DESCRIP:         Settaggi di display GLCD 128*64
/* ============================================================================================================================================ */

//#include "delay.c"

// font e immagini inclusi nel progetto, commentare i font non utilizzati
#include    "Font_4x5.h"
//#include    "Font_5x7.h"
#include    "Font_7x7.h"
//#include    "Font_9x15.h"
//#include    "Font_12x19.h"
//#include    "Font_13x24.h"
//#include    "Font_18x26.h"
#include    "Font_20x31.h"
//#include    "L.h"

/* CONFIGURAZIONE HW ========================================================================================================================== */
//#define FREEDOM
//#define DEDICATED_HW

/* COSTANTI PIN =============================================================================================================================== */
#ifdef FREEDOM
#define GLCD_DATA_WRITE     LATD
#define GLCD_DATA_READ      PORTD
#define GLCD_DATA_DIR       TRISD

#define GLCD_DI             LATAbits.LATA3
#define GLCD_RW             LATAbits.LATA4
#define GLCD_E              LATAbits.LATA5
#define GLCD_CS1            LATEbits.LATE0
#define GLCD_CS2            LATEbits.LATE1
#define GLCD_RST            LATEbits.LATE2
#define GLCD_BACK_ILL       LATCbits.LATC1
#endif

#ifdef DEDICATED_HW
#define GLCD_DATA_WRITE     LATD
#define GLCD_DATA_READ      PORTD
#define GLCD_DATA_DIR       TRISD

#define GLCD_DI             LATAbits.LATA4
#define GLCD_RW             LATAbits.LATA5
#define GLCD_E              LATAbits.LATA3
#define GLCD_CS1            LATEbits.LATE2
#define GLCD_CS2            LATEbits.LATE1
#define GLCD_RST            LATEbits.LATE0
#define GLCD_BACK_ILL       LATCbits.LATC1
#endif

/* COSTANTI GLCD ============================================================================================================================== */
#define INPUT               0b11111111                                                  // costanti per direzione dati PIC I/O
#define OUTPUT              0b00000000
#define X_RESOLUTION        128
#define Y_RESOLUTION        64
#define MAX_X               64
#define MAX_PAGES           8

//#define CS_ACTIVE_HIGH                                                                  // se i CS del GLCD sono attivo alto
#define CS_ACTIVE_LOW                                                                   // se i CS del GLCD sono attivo basso

#ifdef CS_ACTIVE_HIGH
    #define ACTIVE          0b00000001
    #define DEACTIVE        0b00000000
#endif

#ifdef CS_ACTIVE_LOW
    #define ACTIVE          0b00000000
    #define DEACTIVE        0b00000001
#endif

/* COMANDI GLCD =============================================================================================================================== */
#define GLCD_ON             0b00111111
#define GLCD_OFF            0b00111110
#define GLCD_Y_PAGE         0b10111000
#define GLCD_X_ADDRESS      0b01000000
#define GLCD_START_LINE     0b11000000

/* FLAG MASK GLCD ============================================================================================================================= */
#define GLCD_BUSY_FLAG      0b10000000

/* GENERAL CONST ============================================================================================================================== */
#define HIGH                0b00000001
#define LOW                 0b00000000
#define ON                  0b00000001
#define OFF                 0b00000000
#define GLCD_EN_RESET       0b00000000
#define GLCD_DIS_RESET      0b00000001
#define STATUS              0b00000000
#define COMMAND             0b00000000
#define DATA                0b00000001

/* Impostazione delay minimo in base a XTAL_FREQ ============================================================================================== */
#ifndef XTAL_FREQ
    #error Definire XTAL_FREQ
#endif

#if (XTAL_FREQ <= 20)
    #define __delay_minimo();  Nop();
#endif

#if (XTAL_FREQ > 20)
    #define __delay_minimo();  Nop(); Nop();
#endif

/* PROTOTIPI ================================================================================================================================== */

/**
 * Legge il byte di status del GLCD o il dato all'indirizzo settato
 * @param   _DI: indica cosa leggere: STATUS o DATA (unsigned char)
 * @return  ritorna il dato letto (unsigned char)
 * @note    Se si vuole leggere un dato dalla memoria ram Settare il CS, la pagina e l'indirizzo corretti.
 */
unsigned char GLCD_read(unsigned char _DI);

/**
 * Invia i byte di comando o il byte da scricere all'indirizzo settato
 * @param   _DI: indica cosa si sta inviando al GLCD: COMMAND o DATA (unsigned char) \n
 * @param   _data: dato da inviare (unsigned char)
 * @return  void
 * @note   Settare il CS, la pagina e l'indirizzo corretti.
 */
void GLCD_send(unsigned char _DI, unsigned char _data);

/**
 * Attiva o disattiva il GLCD
 * @paran   _status: ON o OFF (unsigned char)
 * @return  void
 */
void GLCD_set_display(unsigned char _status);

/**
 * Seleziona la pagina
 * @param   _page: pagina da selezionare: 0~7 (unsigned char)
 * @return  void
 * @note    Abilitare il CS corretto
 */
void GLCD_set_page(unsigned char _page);

 /**
 * Seleziona l'indirizzo
 * @param   _address: indirizzo x da selezionare: 0~127 (unsigned char)
 * @return  void
 * @note    Abilitare il CS corretto
 */
void GLCD_set_address(unsigned char _address);

/**
 * Attende che il GLCD abbia completato l'istruzione precendente
 * @param   void
 * @return  void
 */
void GLCD_wait_busy(void);

/**
 * Inizializzazione GLCD
 * @param   void
 * @return  void
 */
void GLCD_init(void);

/**
 * Cancella tutti i dati scritti sul GLCD
 * @param   void
 * @return  void
 */
void GLCD_clear(void);

/**
 * Scrive un byte nella pagina selezionata all'indirizzo x selezionato preservando eventuali bit da non sovrascrivere (verifica utilit)
 * Utilizzato in GLCD_write_unsigned char
 * @param   _page: pagina da selezionare: 0~7 (unsigned char) \n
 * @param   _address_x: indirizzo x: 0~127 (unsigned char)    \n
 * @param   _data: byte da scrivere (unsigned char)
 * @return  void
 */
void GLCD_write_byte(unsigned char _page, unsigned char _address, unsigned char _data, unsigned char _on_off);

/**
 * Scrive un bit nell'indirizzo xy selezionato (legge il byte alla pagina che comprende _y all'indirizzo _x, modifica il singolo bit
 * e scrive il byte alla stessa pagina indirizzo _x)
 * @param   _x: indirizzo x: 1~128 (unsigned char)  \n
 * @param   _y: indirizzo y: 1~64 (unsigned char)  \n
 * @param   _on_off: bit acceso o spento: ON o OFF (unsigned char)
 * @return void
 */
void GLCD_write_bit(unsigned char _x, unsigned char _y, unsigned char _on_off);

/**
 * Scrive un immagine salvata in array sul GLCD
 * @param   _image: puntatore al primo indirizzo in flash dell'array contentente l'immagine
 * @return  void
 */
void GLCD_draw_image(const unsigned char  *_image);

/**
 * Disegna una linea orizzontale su GLCD da punto _x_origin a punto _x_dest compresi su riga _y
 * @param   _x_origin: x di partenza: 1~128 (unsigned char)  \n
 * @param   _x_dest: x destinazione: 1~128 (unsigned char)   \n
 * @param   _y: indirizzo y: 1~64 (unsigned char)  \n
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return  void
 */
void GLCD_draw_line_horizontal (unsigned char _x_origin, unsigned char _x_dest, unsigned char _y, unsigned char _on_off);

/**
 * Disegna una linea verticale su GLCD da punto _y_origin a punto _y_dest compresi su colonna _x
 * @param   _y_origin: y di partenza: 1~64 (unsigned char) \n
 * @param   _y_dest: y destinazione: 1~64 (unsigned char)  \n
 * @param   _x: indirizzo x: 1~128 (unsigned char)  \n
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return void
 */
void GLCD_draw_line_vertical (unsigned char _y_origin, unsigned char _y_dest, unsigned char _x, unsigned char _on_off);

/**
 * Disegna una linea da punto a punto su GLCD da punto _x/_y_origin a punto x/_y_dest compresi
 * @param   _x_origin: y di partenza: 1~128 (unsigned char)  \n
 * @param   _y_origin: y di partenza: 1-64 (unsigned char)   \n
 * @param   _x_dest: x di destinazione 1-128 (unsigned char) \n
 * @param   _y_dest: y destinazione: 1~64 (unsigned char)    \n
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return void
 */
void GLCD_draw_line_p2p (unsigned char _x_origin, unsigned char _y_origin, unsigned char _x_dest, unsigned char _y_dest, unsigned char _on_off);

/**
 * Disegna una linea da punto origine di lunghezza stabilita con angolo su GLCD
 * @param   _x_origin: x di partenza: 1~128 (unsigned char)  \n
 * @param   _y_origin: y di partenza: 1-64 (unsigned char)   \n
 * @param   _length: lunghezza: 1-128 (unsigned char)        \n
 * @param   _angle: angolo: 0-360 (unsigned int)             \n
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return void
 */
void GLCD_draw_line_angle(unsigned char _x_origin, unsigned char _y_origin, unsigned char _length, unsigned int _angle, unsigned char _on_off);

/**
 * Disegna una linea da punto a punto su GLCD da punto _x/_y_origin a punto x/_y_dest compresi
 * @param   _x_pos: x centro: 1~128 (unsigned char)   \n
 * @param   _y_pos: y centro: 1-64 (unsigned char)    \n
 * @param   _r: raggio 1-128 (unsigned char)          \n
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return void
 */
void GLCD_draw_circle(unsigned char _x_pos, unsigned char _y_pos, unsigned char _r, unsigned char _on_off);
void GLCD_Plot8(unsigned char _x, unsigned char _y, unsigned char _dx, unsigned char _dy, unsigned char _on_off);

/**
 * Disegna una rettangolo su GLCD
 * @param   _x_origin: x di partenza: 1~128 (unsigned char)  \n
 * @param   _x_dest: x destinazione: 1~128 (unsigned char)   \n
 * @param   _y_origin: y di partenza: 1~64 (unsigned char)   \n
 * @param   _y_dest: y destinazione: 1~64 (unsigned char)    \n
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return void
 */
void GLCD_draw_block (unsigned char _x_origin, unsigned char _x_dest, unsigned char _y_origin, unsigned char _y_dest, unsigned char _on_off);

/**
 * Disegna un rettangolo pieno su GLCD compreso tra i punti x y indicati (punti compresi)
 * @param   _x_origin x di partenza: 1~128 (unsigned char)   \n
 * @param   _x_dest   x destinazione: 1~128 (unsigned char)  \n
 * @param   _y_origin y di partenza: 1~64 (unsigned char)    \n
 * @param   _y_dest   y destinazione: 1~64 (unsigned char)   \n
 * @param   _on_off   accende o spegne i pixel: ON o OFF (unsigned char)
 * @return          void
 */
void GLCD_fill_block (unsigned char _x_origin, unsigned char _x_dest, unsigned char _y_origin, unsigned char _y_dest, unsigned char _on_off);

/**
 * Scrive un singlo carattere su GLCD, se string_mode == 1 i parametri del font non vengono letti.     \n
 *          - scrittura font con array a byte verticali: elimina le colonne vuote in ogni byte carattere del font   \n
 *          - gestione separata font a 1, 2, 3 e 4 byte verticali (per i font a 4 byte vengono gestiti solamente i caratteri numerici) e /:. \n
 *          - la scrittura su GLCD viene fatta per pagina e non per bit attraverso la funzione GLCD_write_byte per velocizzare la scrittura.
 * @param   _x_pos: x di partenza: 1~128, pixel pi basso della scritta (unsigned char)    \n
 * @param   _y_pos: y di partenz: 1~64, pixel pi a sinistra della scritta (unsigned char) \n
 * @param   _unsigned character: carettere da scrivere (unsigned char)   \n
 * @param   _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 * @return ritorna il valore di address(x) effettivamente scritti (unsigned char)
 */
unsigned char GLCD_write_char(unsigned char _x_pos, unsigned char _y_pos, unsigned char _character, unsigned char _font_type, unsigned char _on_off);

/**
 * Scrive una stringa di caratteri su GLCD   \n
 * per font con array a byte verticali lo spazio viene codificato con un incremento di _x_pos proporzionale all'ampliezza del font (per permettere la scrittura salva spazio)
 * @param   _x_pos: x di partenza: 1~128, punto pi basso della scritta (unsigned char)  \n
 * @param   _y_pos: y di partenz: 1~64, punto pi basso della scritta (unsigned char)    \n
 * @param   _string: puntatore ad array di caratteri da scrivere, byte finale \0         \n
 * @param   _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 * @return void
 */
void GLCD_write_string (unsigned char _x_pos, unsigned char _y_pos, const unsigned char _string[], unsigned char _font_type, unsigned char _on_off);

/**
 * Scrive un numero unsigned int su GLCD dopo averlo convertuto in array stringa di caratteri ASCII
 * @param   _x_pos: x di partenza: 1~128, punto pi basso della scritta (unsigned char)   \n
 * @param   _y_pos: y di partenz: 1~64, punto pi basso della scritta (unsigned char)     \n
 * @param   _unsint: valore numerico unsigned int  \n
 * @param   _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 * @return void
 */
void GLCD_write_unsint(unsigned char _x_pos, unsigned char _y_pos, unsigned int _unsint, unsigned char _font_type, unsigned char _on_off);

/**
 * Scrive un numero signed int su GLCD dopo averlo convertuto in array stringa di caratteri ASCII
 * @param   _x_pos: x di partenza: 1~128, punto pi basso della scritta (unsigned char)   \n
 * @param   _y_pos: y di partenz: 1~64, punto pi basso della scritta (unsigned char)     \n
 * @param   _sint: valore numerico signed int
 * @param   _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 * @return  void
 */
void GLCD_write_sint(unsigned char _x_pos, unsigned char _y_pos, signed int _sint, unsigned char _font_type, unsigned char _on_off);

/**
 * Scrive un numero codificado in 1 byte packed BCD su GLCD (non salva spazio)
 * @param   _x_pos: x di partenza: 1~128, punto pi basso della scritta (unsigned char)  \n
 * @param   _y_pos: y di partenz: 1~64, punto pi basso della scritta (unsigned char)    \n
 * @param   _pbcd: valore numerico codificato in packed BCD    \n
 * @param   _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 * @return void
 */
void GLCD_write_PBCD(unsigned char _x_pos, unsigned char _y_pos,unsigned char _pbcd, unsigned char _font_type, unsigned char _on_off);

/**
 * Imposta le variabili a seconda del font selezionato
 * @param   _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 * @return  void
 */
void GLCD_set_font_type (unsigned char _font_type);

/**
 * Resetta le variabili a seconda del font selezionato
 * @return  void
 */
void GLCD_reset_font_type (void);

/**
 * Attiva o disattiva la retroilluminazione
 * @param   _on_off: ON o OFF (unsigned char)
 * @return   void
 */
void GLCD_back_illum(unsigned char _on_off);