/* ============================================================================================================================================= /
 * PROJECT:         Orologio GLCD
 * FILE:            Main.h
 * LAST REV:        2.6 DIGITAL
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F4550 su FREEDOM 2
 * CREATED:         01/12/2013
 * UPDATED:         23/12/2014
 *
 * DESCRIP:         Settings
 *
 * ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 * 2.3              12/04/2014          - Inserite posizioni scritta trashold
 * 2.2              11/04/2014          - Inserita variante di selezione hardware
 * 2.0              07/03/2014          - Disposizione sorgente migliorata
 * 1.1              04/12/2013          - Prima release
 * 1.0              01/12/2013          - Inizio sviluppo

/* I/O PIC18F4550 ============================================================================================================================== /
 * RA0:     Ingresso analogico sensore luce
 * RA1:     NC
 * RA2:     Ingresso analogico sensore temperatura
 * RA3:     GLCD_DI
 * RA4:     GLCD_RW
 * RA5:     GLCD_E
 * RB0:     I2C DATA
 * RB1:     I2C CLK
 * RB2:     Segnale interrupt da RTCC
 * RB4:     SW1: Retroilluminazione/Down
 * RB5:     SW2: Stop allarme/Up
 * RB6:     SW3: Setup allarme
 * RB7:     SW4  Setup data e ora
 * RC0:     Buzzer
 * RC1:     LCD retroilluminazione (PWM)
 * PORTD:   Data I/O GLCD
 * RE0:     GLCD_CS1
 * RE1:     GLCD_CS2
 * RE2:     GLCD_RST
 *
/* ============================================================================================================================================ */

/* CONFIGURATION BITS ========================================================================================================================= */

// CONFIG1L
#pragma config PLLDIV = 1                                                               // PLL Prescaler Selection bits (No prescale (4 MHz oscillator input drives PLL directly))
#pragma config CPUDIV = OSC1_PLL2                                                       // System Clock Postscaler Selection bits ([Primary Oscillator Src: /1][96 MHz PLL Src: /2])
#pragma config USBDIV = 1                                                               // USB Clock Selection bit (used in Full-Speed USB mode only; UCFG:FSEN = 1) (USB clock source comes directly from the primary oscillator block with no postscale)

// CONFIG1H
#pragma config FOSC = HS                                                                // Oscillator Selection bits (HS oscillator (HS))
#pragma config FCMEN = OFF                                                              // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF                                                               // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config PWRT = ON                                                                // Power-up Timer Enable bit (PWRT enabled)
#pragma config BOR = OFF                                                                // Brown-out Reset Enable bits (Brown-out Reset disabled in hardware and software)
#pragma config BORV = 3                                                                 // Brown-out Reset Voltage bits (Minimum setting)
#pragma config VREGEN = OFF                                                             // USB Voltage Regulator Enable bit (USB voltage regulator disabled)

// CONFIG2H
#pragma config WDT = OFF                                                                // Watchdog Timer Enable bit (WDT disabled (control is placed on the SWDTEN bit))
#pragma config WDTPS = 32768                                                            // Watchdog Timer Postscale Select bits (1:32768)

// CONFIG3H
#pragma config CCP2MX = ON                                                              // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF                                                             // PORTB A/D Enable bit (PORTB<4:0> pins are configured as digital I/O on Reset)
#pragma config LPT1OSC = OFF                                                            // Low-Power Timer 1 Oscillator Enable bit (Timer1 configured for higher power operation)
#pragma config MCLRE = ON                                                               // MCLR Pin Enable bit (MCLR pin enabled; RE3 input pin disabled)

// CONFIG4L
#pragma config STVREN = ON                                                              // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
#pragma config LVP = OFF                                                                // Single-Supply ICSP Enable bit (Single-Supply ICSP disabled)
#pragma config ICPRT = OFF                                                              // Dedicated In-Circuit Debug/Programming Port (ICPORT) Enable bit (ICPORT disabled)
#pragma config XINST = OFF                                                              // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

// CONFIG5L
#pragma config CP0 = OFF                                                                // Code Protection bit (Block 0 (000800-001FFFh) is not code-protected)
#pragma config CP1 = OFF                                                                // Code Protection bit (Block 1 (002000-003FFFh) is not code-protected)
#pragma config CP2 = OFF                                                                // Code Protection bit (Block 2 (004000-005FFFh) is not code-protected)
#pragma config CP3 = OFF                                                                // Code Protection bit (Block 3 (006000-007FFFh) is not code-protected)

// CONFIG5H
#pragma config CPB = OFF                                                                // Boot Block Code Protection bit (Boot block (000000-0007FFh) is not code-protected)
#pragma config CPD = OFF                                                                // Data EEPROM Code Protection bit (Data EEPROM is not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF                                                               // Write Protection bit (Block 0 (000800-001FFFh) is not write-protected)
#pragma config WRT1 = OFF                                                               // Write Protection bit (Block 1 (002000-003FFFh) is not write-protected)
#pragma config WRT2 = OFF                                                               // Write Protection bit (Block 2 (004000-005FFFh) is not write-protected)
#pragma config WRT3 = OFF                                                               // Write Protection bit (Block 3 (006000-007FFFh) is not write-protected)

// CONFIG6H
#pragma config WRTC = OFF                                                               // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) are not write-protected)
#pragma config WRTB = OFF                                                               // Boot Block Write Protection bit (Boot block (000000-0007FFh) is not write-protected)
#pragma config WRTD = OFF                                                               // Data EEPROM Write Protection bit (Data EEPROM is not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF                                                              // Table Read Protection bit (Block 0 (000800-001FFFh) is not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF                                                              // Table Read Protection bit (Block 1 (002000-003FFFh) is not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF                                                              // Table Read Protection bit (Block 2 (004000-005FFFh) is not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF                                                              // Table Read Protection bit (Block 3 (006000-007FFFh) is not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF                                                              // Boot Block Table Read Protection bit (Boot block (000000-0007FFh) is not protected from table reads executed in other blocks)

/* CONFIGURAZIONE HW ========================================================================================================================== */
//#define FREEDOM
#define DEDICATED_HW

/* CONSTANTI ==================================================================================================================================== */
#define XTAL_FREQ           20                                                          // definisco la frequenza per Libreria Delay

#ifdef FREEDOM
    #define BUZZ                LATCbits.LATC0
    #define ALARM_INT           PORTBbits.RB2                                           // interrupt allarme da RTCC
    #define SW1                 (unsigned char *)&PORTB, 0b00010000                     // puntatore con valore l'indirizzo di PORTB, maschera bit 4 -> RB4
    #define SW2                 (unsigned char *)&PORTB, 0b00100000                     // puntatore con valore l'indirizzo di PORTB, maschera bit 5 -> RB5
    #define SW3                 (unsigned char *)&PORTB, 0b01000000                     // puntatore con valore l'indirizzo di PORTB, maschera bit 6 -> RB6
    #define SW4                 (unsigned char *)&PORTB, 0b10000000                     // puntatore con valore l'indirizzo di PORTB, maschera bit 7 -> RB7
#endif

#ifdef DEDICATED_HW
    #define BUZZ                LATCbits.LATC2
    #define ALARM_INT           PORTBbits.RB2                                           // interrupt allarme da RTCC
    #define SW1                 (unsigned char *)&PORTB, 0b00001000
    #define SW2                 (unsigned char *)&PORTB, 0b00010000
    #define SW3                 (unsigned char *)&PORTB, 0b00100000
    #define SW4                 (unsigned char *)&PORTC, 0b00000001
#endif

//#define ON                  1
//#define OFF                 0

#define DUTY_LOW_NIGHT      2                                                           // valori duty cicle retroilluminazione
#define DUTY_LOW_DAY        50
#define DUTY_HIGH_NIGHT     30
#define DUTY_HIGH_DAY       200

#define BACK_ON             5                                                           // secondi in cui la retroilluminazione resta accesa

#ifdef FREEDOM
    #define LM35_OFFSET         3                                                       
#endif

#ifdef DEDICATED_HW
    #define LM35_OFFSET         77                                                      // offset temperatura in BCD
#endif


/* POSIZIONI SCRITTE ============================================================================================================================ */
#define HOUR_X              15                                                          // cambiando la posizione della scritta si sposta anche il relativo BOX
#define HOUR_Y              35                                                          // a patto di mantenere lo stesso font
#define MIN_X               71
#define MIN_Y               35
#define DOTS_X              54
#define DOTS_Y              35
#define DAY_X               6
#define DAY_Y               60
#define MONTH_X             31
#define MONTH_Y             60
#define YEAR_20_X           55
#define YEAR_X              70
#define YEAR_Y              60
#define FIRST_SP_X          22
#define FIRST_SP_Y          60
#define SECOND_SP_X         47
#define SECOND_SP_Y         60
#define DAY_WE_X            6
#define DAY_WE_Y            50
#define AL_HOUR_X           15
#define AL_HOUR_Y           35
#define AL_MIN_X            71
#define AL_MIN_Y            35
#define AL_DOTS_X           54
#define AL_DOTS_Y           35
#define AL_DAY_X            20
#define AL_DAY_Y            48
#define AL_DAY_M_X          20
#define AL_DAY_M_Y          57
#define AL_WRITE_X          100
#define AL_WRITE_Y          43
#define AL_ON_X             100
#define AL_ON_Y             49
#define TONE_X              100
#define TONE_Y              55
#define TONE_ON_X           100
#define TONE_ON_Y           61
#define TEMP_X              89
#define TEMP_Y              60
#define TRESHOLD_X          50
#define TRESHOLD_Y          30

//BOX
#define BLOCK_HOUR          HOUR_X-1, HOUR_X+42, HOUR_Y-31, HOUR_Y+1
#define BLOCK_MIN           MIN_X-1, MIN_X+42, MIN_Y-31, MIN_Y+1
#define BLOCK_DAY           DAY_X-1, DAY_X+14, DAY_Y-7,DAY_Y+1
#define BLOCK_MONTH         MONTH_X-1, MONTH_X+14, MONTH_Y-7, MONTH_Y+1
#define BLOCK_YEAR          YEAR_20_X-1, YEAR_20_X+29, YEAR_Y-7, YEAR_Y+1
#define BLOCK_DAY_W         DAY_WE_X-1, DAY_WE_X+61, DAY_WE_Y-7 ,DAY_WE_Y+1
#define BLOCK_AL_HOUR       AL_HOUR_X-1,AL_HOUR_X+42,AL_HOUR_Y-31, AL_HOUR_Y+1
#define BLOCK_AL_MIN        AL_MIN_X-1, AL_MIN_X+42,AL_MIN_Y-31, AL_MIN_Y+1
#define BLOCK_AL_WR         AL_WRITE_X-1, AL_WRITE_X+24,AL_WRITE_Y-5,AL_WRITE_Y+1
#define BLOCK_AL_ON         AL_ON_X-1, AL_ON_X+11, AL_ON_Y-5, AL_ON_Y+1
#define BLOCK_TONE_ON       TONE_ON_X-1, TONE_ON_X+11, TONE_ON_Y-5, TONE_ON_Y+1
#define BLOCK_TEMP          TEMP_X-1, TEMP_X+37, TEMP_Y-7, TEMP_Y+1
#define BLOCK_TRESHOLD      TRESHOLD_X, TRESHOLD_X+50,TRESHOLD_Y-7,TRESHOLD_Y
#define BLOCK_AL_DAY_X1     AL_DAY_M_X-1
#define BLOCK_AL_DAY_X2     AL_DAY_M_X+7
#define BLOCK_AL_DAY_Y      AL_DAY_M_Y-7,AL_DAY_M_Y+1

/* GIORNI ======================================================================================================================================= */
#define D0                  "Domenica "                                                 
#define D1                  "Lunedi"
#define D2                  "Martedi"
#define D3                  "Mercoledi"
#define D4                  "Giovedi"
#define D5                  "Venerdi"
#define D6                  "Sabato"

/* RTCC VALORI MINIMI E MASSIMI ================================================================================================================ */
#define MIN_SEC             0x00                                                        // valori minimi e massimi per ogni dato
#define MAX_SEC             0x59
#define MIN_MIN             0x00
#define MAX_MIN             0x59
#define MIN_HOU             0x00
#define MAX_HOU             0x23
#define MIN_DAY             0x01
#define MAX_DAY             0x31
#define MIN_MONTH           0x01
#define MAX_MONTH           0x12
#define MIN_YEAR            0x00
#define MAX_YEAR            0x99
#define MAX_DAY_WEEK        0x06

/* ============================================================================================================================================= /
 * FUNCTION:        DefineIO
 * Overview:        Definizione e inizializzazione ingressi e uscite
/* ============================================================================================================================================ */

void Define_IO(void) {

#ifdef FREEDOM
    LATA =      0b00000000;                                                             // Reset Latch uscite
    TRISA =     0b00000101;
    LATB =      0b00000000;                                                             // Reset Latch uscite
    TRISB =     0b11110101;
    LATC =      0b00000000;                                                             // Reset Latch uscite
    TRISC =     0b11000000;
    LATD =      0b00000000;                                                             // Reset Latch uscite
    TRISD =     0b00000000;
    LATE = 0x00;
    TRISE = 0xF8;
#endif

#ifdef DEDICATED_HW
    LATA =      0b00000000;                                                             // Reset Latch uscite
    TRISA =     0b00000101;
    LATB =      0b00000000;                                                             // Reset Latch uscite
    TRISB =     0b11111101;
    LATC =      0b00000000;                                                             // Reset Latch uscite
    TRISC =     0b11000001;
    LATD =      0b00000000;                                                             // Reset Latch uscite
    TRISD =     0b00000000;
    LATE =      0b00000000;
    TRISE =     0b11111000;
#endif
}

/* ============================================================================================================================================= /
 * FUNCTION:        Module_settings
 * Overview:        Configurazioni dei moduli interni
/* ============================================================================================================================================ */

void Module_settings(void) {
    UCON =      0;                                                                      // disabilito USB
    UCFG =      1;

    INTCON2bits.RBPU = 0;                                                               // abilito resistenze pull up su PORTB

    CCP1CONbits.CCP1M0 = 0;                                                             // disabilito tutti i comparatori
    CCP1CONbits.CCP1M1 = 0;
    CCP1CONbits.CCP1M2 = 0;
    CCP1CONbits.CCP1M3 = 0;

    CCP2CONbits.CCP2M0 = 0;
    CCP2CONbits.CCP2M1 = 0;
    CCP2CONbits.CCP2M2 = 0;
    CCP2CONbits.CCP2M3 = 0;

    INTCONbits.GIE = 1;                                                                 // abilito global interrupt
    INTCONbits.PEIE = 1;                                                                // abilito interrupt periferiche
    INTCONbits.TMR0IE = 1;                                                              // abilito interrupt timer 0
    PIE1bits.TMR1IE = 1;                                                                // abilito interrupt timer 0
    INTCON3bits.INT2IE = 1;                                                             // abilito interrupt INT0
    INTCON2bits.INTEDG2 = 0;                                                            // INT2 su falling edge

    // TIMER 0, Configurazioni per avere 1 interrupt ogni 250ms = (65536-preload)*(4/fosc)*prescaler = (65536-26476)*(4/20000000)*32
    T0CONbits.T0PS0 =0;                                                                 // prescaler 1/32
    T0CONbits.T0PS1 =0;                                                                 //
    T0CONbits.T0PS2 =1;
    T0CONbits.PSA = 0;                                                                  // prescaler assegnato
    T0CONbits.T0CS = 0;                                                                 // da clock interno fosc/4
    T0CONbits.T08BIT = 0;                                                               // 16 BIT
    TMR0 = 26476;                                                                       // preload timer 0

    T0CONbits.TMR0ON = 0;

    // TIMER 1, Configurazioni per avere 1 interrupt ogni 100ms = (65536-preload)*(4/fosc)*prescaler = (65536-3038)*(4/20000000)*8
    T1CONbits.T1CKPS0 = 1;                                                              // prescaler 1/8
    T1CONbits.T1CKPS1 = 1;
    T1CONbits.TMR1CS = 0;                                                               // da clock interno fosc/4
    T1CONbits.RD16 = 1;                                                                 // 16 BIT
    TMR1 = 3038;                                                                        // preload timer 1

    T1CONbits.TMR1ON = 0;

    // Configurazioni per ADC
    ADCON1 =    0b00001100; //0b00001101                                                // Abilito AN0-AN1-AN2 come ingressi analogici VREF sono impostate a massa e VCC
    ADCON2 =    0b10110100;                                                             // TAD : FOSC/4, TACQ: 16 TAD
    ADCON0bits.ADON = 0;                                                                // disabilito l' ADC

    // Configurazioni per PWM

    T2CONbits.TMR2ON = 1;
    T2CONbits.T2CKPS0 = 0;                                                              // prescaler 16
    T2CONbits.T2CKPS1 = 1;
    T2CONbits.T2OUTPS0 = 1;
    T2CONbits.T2OUTPS1 = 1;
    T2CONbits.T2OUTPS2 = 1;
    T2CONbits.T2OUTPS3 = 1;
    PR2 = 249;                                                                          // imposto il periodo

    CCP2CONbits.CCP2M = 1;

    CCP2CONbits.CCP2M2 = 1;                                                             //attivo PWM2
    CCP2CONbits.CCP2M3 = 1;
}

