/* ============================================================================================================================================= /
 * LIBRARY:         RTCC PCF8563
 * FILE:            RTCC_PCF8563.h
 * LAST REV:        1.2
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         02/12/2013
 * UPDATED:         03/12/2013
 *
 * DESCRIP:         Settaggi per RTCC_PCF8563.c
/* ============================================================================================================================================ */

/* IMPOSTAZIONI =============================================================================================================================== */
#define WRITE_ADDR                      0xA2                                            // indirizzi PCF8563

#define CONTROL_REG_ADDR                0x00
#define CONTROL_REG_2_ADDR              0x01
#define SECONDS_ADDR                    0x02
#define MINUTES_ADDR                    0x03
#define HOURS_ADDR                      0x04
#define DAY_ADDR                        0x05
#define DAY_WEEK_ADDR                   0x06
#define MONTH_ADDR                      0x07
#define YEAR_ADDR                       0x08
#define MINUTES_ALARM_ADDR              0x09
#define HOURS_ALARM_ADDR                0x0A
#define DAYS_ALARM_ADDR                 0x0B
#define DAY_WEEK_ALARM_ADDR             0x0C
#define CLOCKOUT_REG_ADR                0x0D

/* COSTANTII ================================================================================================================================== */

#define AL_ON                           0b00000000
#define AL_OFF                          0b10000000

#define CLK_OFF                         0b00000000
#define CLK_32768_Hz                    0b10000000
#define CLK_1024_Hz                     0b10000001
#define CLK_32_Hz                       0b10000010
#define CLK_1_Hz                        0b10000011


/* PROTOTIPI ================================================================================================================================== */

/**
 * Legge i secondi dal RTCC
 * @param   void
 * @return  torna il valore dei secondi codificato in packed BCD
 */
unsigned char RTCC_read_seconds(void);

/**
 * Legge i minuti dal RTCC
 * @param   void
 * @return  torna il valore dei minuti codificato in packed BCD
 */
unsigned char RTCC_read_minutes(void);

/**
 * Legge le ore dal RTCC
 * @param   void
 * @return  torna il valore dele ore codificato in packed BCD
 */
unsigned char RTCC_read_hours(void);

/**
 * Legge il giorno dal RTCC
 * @param   void
 * @return  torna il valore del giorno codificato in packed BCD
 */
unsigned char RTCC_read_day(void);

/**
 * Legge il mese dal RTCC
 * @param   void
 * @return  torna il valore del mese codificato in packed BCD
 */
unsigned char RTCC_read_month(void);

/**
 * Legge l'anno dal RTCC
 * @param   void
 * @return  torna il valore dell'anno codificato in packed BCD
 */
unsigned char RTCC_read_year(void);

/**
 * Legge il giorno della settimana dal RTCC
 * @param   void
 * @return  torna il valore del giorno della settimana
 */
unsigned char RTCC_read_day_w(void);

/**
 * Legge i minuti dell'allarme dal RTCC
 * @param   void
 * @output  torna il valore dei minuti dell'allarme codificato in packed BCD
 */
unsigned char RTCC_read_minutes_al(void);

/**
 * Legge le ore dell'allarme dal RTCC
 * @param   void
 * @return  torna il valore delle ore dell'allarme codificato in packed BCD
 */
unsigned char RTCC_read_hours_al(void);

/**
 * Legge il giorno dell'allarme dal RTCC
 * @param   void
 * @return  torna il valore del giorno dell'allarme codificato in packed BCD
 */
unsigned char RTCC_read_day_al(void);

/**
 * Legge il giorno della settimana dell'allarme dal RTCC
 * @param   void
 * @return  torna il valore del giorno della settimana dell'allarme codificato in packed BCD
 */
unsigned char RTCC_read_day_w_al(void);

/**
 * Scrive il valore dei secondi nell' RTCC
 * @param   _seconds: valore dei secondi da scrivere nell' RTCC codificato in packed BCD
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error  \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision   \n
 */
signed char RTCC_write_seconds(unsigned char _seconds);

/**
   Scrive il valore dei minuti nell' RTCC
 * @param   _minutes : valore dei minuti da scrivere nell' RTCC codificato in packed BCD
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error  \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision   \n
 */
signed char RTCC_write_minutes(unsigned char _minutes);

/**
 * Scrive il valore delle ore nell' RTCC
 * @param   _hours : valore delle ore da scrivere nell' RTCC codificato in packed BCD
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error  \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision   \n
 */
signed char RTCC_write_hours(unsigned char _hours);

/**
 * Scrive il valore del giorno nell' RTCC
 * @param   _day : valore del giorno da scrivere nell' RTCC codificato in packed BCD
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error  \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision   \n
 */
signed char RTCC_write_day(unsigned char _day);

/**
 * Scrive il valore del mese nell' RTCC
 * @param   _day : valore del mese da scrivere nell' RTCC codificato in packed BCD
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error  \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision   \n
 */
signed char RTCC_write_month(unsigned char _month);

/**
 * Scrive il valore dell'anno nell' RTCC
 * @param   _year : valore dell'anno da scrivere nell' RTCC codificato in packed BCD
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error  \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision   \n
 */
signed char RTCC_write_year(unsigned char _year);

/**
 * Scrive il valore del giorno della settimana nell' RTCC
 * @param   _day_w : valore del giorno della settimana da scrivere nell' RTCC codificato in packed BCD
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error  \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision   \n
 */
signed char RTCC_write_day_w(unsigned char _day_w);

/**
 * Scrive il valore dei minuti dell'allarme nell' RTCC
 * @param   _minutes_al : valore dei minuti dell'allarme da scrivere nell' RTCC codificato in packed BCD
 * @param   _enable: abilitazione o disabilitazione dell'allarme (AL_ON, AL_OFF)
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error   \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision    \n
 */
signed char RTCC_write_minutes_al(unsigned char _minutes_al, unsigned char _enable);

/**
 * Scrive il valore delle ore dell'allarme nell' RTCC
 * @param   _hours_al : valore delle ore dell'allarme da scrivere nell' RTCC codificato in packed BCD
 * @param   _enable: abilitazione o disabilitazione dell'allarme (AL_ON, AL_OFF)
 * @return  1: The info has been properly written  \n
 *          -1: Bus Collision error   \n
 *          -2: Not Ack error condition  \n
 *          -3: Write collision
 */
signed char RTCC_write_hours_al(unsigned char _hours_al, unsigned char _enable);

/**
 * Scrive il valore del giorno dell'allarme nell' RTCC
 * @param   _day_al : valore del giorno dell'allarme da scrivere nell' RTCC codificato in packed BCD
 * @param   _enable: abilitazione o disabilitazione dell'allarme (AL_ON, AL_OFF)
 * @return  1: The info has been properly written  \n
 *          -1: Bus Collision error   \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision
 */
signed char RTCC_write_day_al(unsigned char _day_al, unsigned char _enable);

/**
 * Scrive il valore del giorno della settimana dell'allarme nell' RTCC
 * @param   _day_w_al : valore del giorno della settimana dell'allarme da scrivere nell' RTCC codificato in packed BCD
 * @param   _enable: abilitazione o disabilitazione dell'allarme (AL_ON, AL_OFF)
 * @return  1: The info has been properly written  \n
 *          -1: Bus Collision error   \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision
 */
signed char RTCC_write_day_w_al(unsigned char _day_w_al, unsigned char _enable);

/**
 * Attiva o disattiva l'interrupt di allarme avvenuto a seconda dell'ingresso
 * @param   _flag : Attiva o disattiva l'interrupt di allarme (ON (1), OFF(2))
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error  \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision   \n
 */
signed char RTCC_set_alarm_interrupt(unsigned char _flag);

/**
 * Resetta flag di allarme avvenuto nel RTCC
 * @param   void
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error  \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision   \n
 */
signed char RTCC_clear_alarm(void);

/**
 * Imposta o disattiva il clock in uscita.
 * I@param  _setting: settaggio del clock in uscita: CLK_OFF, 32.768_KHz, 1.024_KHz, 32_Hz, 1_Hz
 * @return  1: The info has been properly written   \n
 *          -1: Bus Collision error  \n
 *          -2: Not Ack error condition   \n
 *          -3: Write collision   \n
 */
signed char RTCC_clock_out(unsigned char _setting);
