/* ============================================================================================================================================ */
/* LIBRARY:         Lettura e scrittura da EEPROM interna
/* FILE:            EEPROM.c
/* LAST REV:        1.0
/* AUTHOR:          Ivo Colleoni
/* DEVICE:          PIC18F
/* CREATED:         24/09/2013
/* UPDATED:         24/09/2013
 *
 * DESCRIP:         Settaggi per modulo SPI
/* ============================================================================================================================================ */

/* PROTOTIPI ================================================================================================================================== */

/**
 * Restituisce il valore contenuto nell'indirizzo dichiarato
 * @param   _adr: indirizzo da leggere (unsigned char)
 * @return  valore letto (unsigned char)
 */
unsigned char EEPROM_read(unsigned char _adr);

/**
 * Salva il valore nell'indirizzo dichiarato
 * @param   _data: valore da scrivere (unsigned char)
 * @param   _adr: indirizzo in cui scrivere (unsigned char)
 * @return  void
 */
void EEPROM_write(unsigned char _data, unsigned char _adr);

/**
 * Restituisce un valore intero da -32768 a 32767 letto da 2 indirizzi consecutivi
 * @param   _adr: indirizzo in cui scrivere (unsigned char)
 * @return  valore letto in _adr e _adr+1 (signed int)
    */
int EEPROM_read_int(unsigned char _adr);

/**
 * Salva il valore intero da -32768 a 32767 nell'indirizzo dichiarato e in quello successivo (2 byte)
 * @param   _data: valore da scrivere (signed int)
 * @param   _adr: indirizzo in cui scrivere (unsigned char)
 * @return  void
 */
void EEPROM_write_int(int _data, unsigned char _adr);
